﻿// This file contains your Data Connector logic
[Version = "1.0.0"]
section Reftab;

// Data Source Kind description
Reftab = [
    Authentication = [
        // Key = [],
        UsernamePassword = [Label = "Reftab API",UsernameLabel = "Public Key", PasswordLabel = "Secret Key"]
        // Windows = [],
        //Implicit = []
    ],
    Label = Extension.LoadString("DataSourceLabel")
];

// Data Source UI publishing description
Reftab.Publish = [
    Beta = true,
    Category = "Other",
    ButtonText = { Extension.LoadString("ButtonTitle"), Extension.LoadString("ButtonHelp") },
    LearnMoreUrl = "https://www.reftab.com/",
    SourceImage = Reftab.Icons,
    SourceTypeImage = Reftab.Icons
];

Reftab.Icons = [
    Icon16 = { Extension.Contents("Reftab16.png"), Extension.Contents("Reftab20.png"), Extension.Contents("Reftab24.png"), Extension.Contents("Reftab32.png") },
    Icon32 = { Extension.Contents("Reftab32.png"), Extension.Contents("Reftab40.png"), Extension.Contents("Reftab48.png"), Extension.Contents("Reftab64.png") }
];

BaseUrl = "https://www.reftab.com/api/";

[DataSource.Kind="Reftab", Publish="Reftab.Publish"]
shared Reftab.Contents = ReftabNavTable;

ReftabNavTable = () =>
    let
        source = #table({"Name", "Data", "ItemKind", "ItemName", "IsLeaf"}, {
            { "Assets", Reftab.Fetch("assets"), "Table", "Table", true },
            { "Loans", Reftab.Fetch("loans"), "Table", "Table", true }
        }),
        navTable = Table.ToNavigationTable(source, {"Name"}, "Name", "Data", "ItemKind", "ItemName", "IsLeaf")
    in
        navTable;

Reftab.Fetch = (endpoint as text) =>
    let
        status = if endpoint = "loans" then "&status=all" else "",
        creds = Extension.CurrentCredential(),
        url = Text.Combine({BaseUrl, endpoint, "?limit=100000", status}),
        now = DateTimeZone.ToText(DateTimeZone.UtcNow(), "yyyy-MM-ddTHH:mm:ssZ"),
        signatureToSign = Text.Combine({"GET#(lf)#(lf)#(lf)", now, "#(lf)", url}),
        textBin = Text.ToBinary(signatureToSign),
        secret = Text.ToBinary(creds[Password]),
        hmac = Binary.ToText(Crypto.CreateHmac(CryptoAlgorithm.SHA256, secret, textBin), BinaryEncoding.Hex),
        hmac64 = Binary.ToText(Text.ToBinary(hmac), BinaryEncoding.Base64),
        resp = Web.Contents(url, [ManualStatusHandling={},Headers = [
            Authorization = Text.Combine({"RT ", creds[Username], ":", hmac64}),
            #"x-rt-date" = now
        ]]),
        json = Table.FromRecords(Json.Document(resp), null, MissingField.UseNull)
    in
        json;

Table.ToNavigationTable = (
    table as table,
    keyColumns as list,
    nameColumn as text,
    dataColumn as text,
    itemKindColumn as text,
    itemNameColumn as text,
    isLeafColumn as text
) as table =>
    let
        tableType = Value.Type(table),
        newTableType = Type.AddTableKey(tableType, keyColumns, true) meta 
        [
            NavigationTable.NameColumn = nameColumn, 
            NavigationTable.DataColumn = dataColumn,
            NavigationTable.ItemKindColumn = itemKindColumn, 
            Preview.DelayColumn = itemNameColumn, 
            NavigationTable.IsLeafColumn = isLeafColumn
        ],
        navigationTable = Value.ReplaceType(table, newTableType)
    in
        navigationTable;